;;This is an agent-based computer model of the labor market. This is mainly used to study the impact of expanding the scope of
;; workers'job search. This model includes several subroutines, such as job search, wage negotiation and variable update. This
;; model needs to be run in Netlogo program. Netlogo is a software for multi-agent simulation modeling. Software download
;; link: http://ccl.northwestern.edu/netlogo/


;;Workers'color: blue means employment; yellow means no vacancies within the scope; 
;;white means no resources to continue searching; green means that the expected earnings of employment are lower than unemployment benefits; 
;;pink means that wages are too low after negotiations; black means that all applicants are rejected; and orange means that the search has not started.


;;Create variables
globals[price old_price growth_rate dieout GDP rate_of_unemployed yellow_rate green_rate pink_rate white_rate black_rate orange_rate wage_gap insurance insurance-old]
breed[firms firm]
breed[workers worker]
firms-own[job n gain gain-t wage wage-s product energy power_of_firm net-wage hereworker his-r]      
workers-own[w w-s range his-wage refuse-time search-time wallet accepted_rate his-blue search-cost d blue-history black-history yellow-history white-history green-history pink-history]

;;Initial settings:Workers and firms are randomly distributed around the world, and initial variables are set.
to setup
 clear-all
 create-firms num-firms                                                    
 ask firms[
 set xcor random 35 + 1
 set ycor random 35 + 1
 while [count firms-here >= 2]
 [fd 1]
 set shape "house"
 set size 2
 set color red  
 set energy 100                                                            
 set job int(1 + abs random-normal (num-workers / num-firms)  num-workers / num-firms )          
 set wage-s int(abs random-normal 100 10)                                  
 set wage 100
 set his-r [0 0]]                                                               
 create-workers num-workers[
     setxy random-xcor random-ycor                                   
     set size 1                                                 
     set color orange                                                        
     set w-s int(abs random-normal 100 10)
     set his-wage 0                               
     set range search_scope                                                    
     set accepted_rate (7 + random 3) / 10 
     set wallet 100 * 0.45
     set his-blue [0 0]
     set blue-history 0
     set green-history 0
     set black-history 0
     set white-history 0
     set pink-history 0
     set yellow-history 0
     ]                       
 set price 100                                                    
 set insurance price * 0.45
end

to go
tick
search-job
negotiation
if ticks >= 200
[record-color]
profile
do-plot                                                            
update_wage_satisfing
die_out
ask workers[
    set color orange
    setxy random-xcor random-ycor ]
ask firms[
set xcor random 35 + 1
set ycor random 35 + 1
while [count firms-here >= 2]
 [fd 1]
]
if ticks = 500
;;The results of different types of unemployment rate of workers evaluated by different markets are shown.
[show sum [blue-history] of workers with [accepted_rate = 0.7] / count workers with [accepted_rate = 0.7]
 show sum [yellow-history] of workers with [accepted_rate = 0.7] / count workers with [accepted_rate = 0.7]
 show sum [green-history] of workers with [accepted_rate = 0.7] / count workers with [accepted_rate = 0.7]
 show sum [pink-history] of workers with [accepted_rate = 0.7] / count workers with [accepted_rate = 0.7]
 show sum [black-history] of workers with [accepted_rate = 0.7] / count workers with [accepted_rate = 0.7]
 show sum [white-history] of workers with [accepted_rate = 0.7] / count workers with [accepted_rate = 0.7]
 show sum [blue-history] of workers with [accepted_rate = 0.8] / count workers with [accepted_rate = 0.8]
 show sum [yellow-history] of workers with [accepted_rate = 0.8] / count workers with [accepted_rate = 0.8]
 show sum [green-history] of workers with [accepted_rate = 0.8] / count workers with [accepted_rate = 0.8]
 show sum [pink-history] of workers with [accepted_rate = 0.8] / count workers with [accepted_rate = 0.8]
 show sum [black-history] of workers with [accepted_rate = 0.8] / count workers with [accepted_rate = 0.8]
 show sum [white-history] of workers with [accepted_rate = 0.8] / count workers with [accepted_rate = 0.8]
 show sum [blue-history] of workers with [accepted_rate = 0.9] / count workers with [accepted_rate = 0.9]
 show sum [yellow-history] of workers with [accepted_rate = 0.9] / count workers with [accepted_rate = 0.9]
 show sum [green-history] of workers with [accepted_rate = 0.9] / count workers with [accepted_rate = 0.9]
 show sum [pink-history] of workers with [accepted_rate = 0.9] / count workers with [accepted_rate = 0.9]
 show sum [black-history] of workers with [accepted_rate = 0.9] / count workers with [accepted_rate = 0.9]
 show sum [white-history] of workers with [accepted_rate = 0.9] / count workers with [accepted_rate = 0.9]
 stop]
end

;;Job search of workers
to search-job
ask workers[
  create-links-with firms
  ask link-neighbors [set hereworker count workers-here with [color = blue]]
  ask links [die]
  set refuse-time 0
  set search-time 0
  set search-cost 0
  set wallet insurance
  create-links-with firms with [distance myself <= [range] of myself and hereworker < job]
  if count link-neighbors = 0
  [set color yellow
   set w 0
   set xcor 0
   set ycor 0]
  ask link-neighbors [set net-wage wage-s * [accepted_rate] of myself - distance myself * transport-price * insurance]
  while [count link-neighbors >= 1 and color = orange]
    [set d distance max-one-of link-neighbors [net-wage]
     ifelse (wallet >= d * transport-price * insurance) and ([net-wage] of max-one-of link-neighbors [net-wage] >= insurance)
      [set wallet wallet - (d * transport-price * insurance)
       set search-cost d + search-cost
       set search-time search-time + 1
       move-to max-one-of link-neighbors [net-wage]
       ifelse random-float 1 < accepted_rate
         [set color  blue
          stop]
         [set refuse-time refuse-time + 1
          ask my-links with [link-length = 0] [die]
          if count link-neighbors = 0
          [set color black
           set w 0
           set xcor 0
           set ycor 0
           stop]]]
      [if wallet < d * transport-price * insurance
         [set color white
          set w 0
          set xcor 0
          set ycor 0
          ask my-links [die]
          stop]
       if [net-wage] of max-one-of link-neighbors [net-wage] < insurance
          [set color green
           set w 0
           set xcor 0
           set ycor 0
           ask my-links [die]
           stop]
        ]
        set d 0]]
       
ask firms[
    set n count workers-here with[color = blue]]                                         
ask links [die]
end


;;Negotiation about wage
to negotiation
ask firms[
    set power_of_firm (n / job)
    ifelse n > 0
    [set wage wage-s * power_of_firm + sum([w-s] of workers-here) / n * (1 - power_of_firm)]
    [set wage 0]]   
ask workers with [color = blue] 
    [if w-s * toleration_ratio > item 0 [wage] of firms-here
    [set color pink                                                  
    set w 0                                                           
    set xcor 0
    set ycor 0]]
ask workers[
    if color = blue[
        set w item 0 [wage] of firms-here ]]                                 
ask firms[
    set n count workers-here]                           
end 


;;Calculating output and wage growth
to profile
ask firms[
    set product n ]                                             
set GDP sum([product] of firms)                                
set insurance-old insurance
set insurance sum([w] of workers) * 0.45 / GDP
set old_price price
set price (sum([w] of workers with [color = blue]) + insurance-old * 500 - 500 * insurance) / GDP    
set growth_rate (price - old_price) / old_price * 100                       
ask firms[
    set gain-t gain
    set gain product * price - wage * n 
    set energy energy + gain]                              
end

;;The elimination mechanism of firms
to die_out
set dieout 0                                                              
ask firms with [energy < 0][
    set dieout dieout + 1
    set job int(abs random-normal (num-workers / num-firms) 20) + 1
    set energy sum([energy] of firms with [energy >= 0]) / count firms with [energy >= 0]
    set wage sum([wage] of firms with [energy >= 0]) / count firms with [energy >= 0]
    set wage-s sum([wage-s] of firms with [energy >= 0]) / count firms with [energy >= 0]
    set gain-t 0
    ]
end


;;Adjustment of expected wages
to update_wage_satisfing
ask firms[
    let r n / job
    set his-r lput r his-r
    set his-r remove-item 0 his-r
    ifelse (item 0 his-r = 1) and (item 1 his-r = 1)
    [set wage-s wage-s * (1 - random-float 0.03)]
    [set wage-s wage-s * (1 + random-float 0.03)]
    if gain > gain-t
    [set job job + 1]
    if gain < gain-t and job > 1
    [set job job - 1]]         
    
ask workers [
    ifelse color = blue
    [set his-blue lput 1 his-blue]
    [set his-blue lput 0 his-blue]
    set his-blue remove-item 0 his-blue
    ifelse his-blue = [1 1]
    [set w-s w-s * (1 + random-float 0.03 )]
    [set w-s w-s * (1 - random-float 0.03)]]
end

to do-plot
set-current-plot "Wage growth rate"
set-current-plot-pen "growth_rate"
  plot growth_rate
  
 let unemployed count workers with [color != blue] 
 set rate_of_unemployed  unemployed / num-workers
 set-current-plot "unemployment rate"
 set-current-plot-pen "unemployment rate"
 plot rate_of_unemployed
 set-current-plot-pen "yellow"
 set yellow_rate count workers with [color = yellow] / num-workers
 plot yellow_rate
 set-current-plot-pen "green"
 set green_rate count workers with [color = green] / num-workers
 plot green_rate
 set-current-plot-pen "pink"
 set pink_rate count workers with [color = pink] / num-workers
 plot pink_rate
   set-current-plot-pen "white"
 set white_rate count workers with [color = white] / num-workers
 plot white_rate
   set-current-plot-pen "black"
 set black_rate count workers with [color = black] / num-workers
 plot black_rate
  set-current-plot-pen "orange"
 set orange_rate count workers with [color = orange] / num-workers
 plot orange_rate
 
 set-current-plot "GDP"
 set-current-plot-pen "GDP"
 plot GDP
 
set-current-plot "Number of positions"
set-current-plot-pen "Number of positions"
plot sum [job] of firms
 
 set-current-plot "vacancy"
 set-current-plot-pen "vacancy"
 plot sum [job] of firms - GDP

end


;;Record each worker's past unemployment type
to record-color
ask workers[
set his-wage his-wage + w
if color = blue
[set blue-history blue-history + 1]
if color = yellow
[set yellow-history yellow-history + 1]
if color = green
[set green-history green-history + 1]
if color = pink
[set pink-history pink-history + 1]
if color = black
[set black-history black-history + 1]
if color = white
[set white-history white-history + 1]
]
end 
@#$#@#$#@
GRAPHICS-WINDOW
201
10
628
458
-1
-1
11.2703
1
14
1
1
1
0
1
1
1
0
36
0
36
0
0
1
ticks

CC-WINDOW
5
650
838
745
Command Center
0

SLIDER
14
10
186
43
num-workers
num-workers
0
1000
500
1
1
NIL
HORIZONTAL

SLIDER
13
49
185
82
num-firms
num-firms
0
100
50
1
1
NIL
HORIZONTAL

BUTTON
204
459
270
492
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
13
82
185
115
toleration_ratio
toleration_ratio
0
1
0.5
0.01
1
NIL
HORIZONTAL

PLOT
3
183
203
333
Wage growth rate
NIL
NIL
0.0
10.0
-2.0
2.0
true
false
PENS
"price" 1.0 0 -16777216 true
"growth_rate" 1.0 0 -16777216 true

PLOT
3
486
203
636
unemployment rate
NIL
NIL
0.0
10.0
0.0
0.5
true
false
PENS
"unemployment rate" 1.0 0 -16777216 true
"yellow" 1.0 0 -1184463 true
"green" 1.0 0 -10899396 true
"pink" 1.0 0 -2064490 true
"orange" 1.0 0 -955883 true
"white" 1.0 0 -1 true
"black" 1.0 0 -16777216 true

PLOT
3
335
203
485
GDP
NIL
NIL
0.0
10.0
0.0
10.0
true
false
PENS
"GDP" 1.0 0 -16777216 true

PLOT
627
165
827
315
vacancy
NIL
NIL
0.0
1.0
0.0
1.0
true
false
PENS
"vacancy" 1.0 0 -16777216 true

SLIDER
11
115
183
148
search_scope
search_scope
5
10
9
1
1
NIL
HORIZONTAL

BUTTON
270
458
335
496
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
12
148
184
181
transport-price
transport-price
0.01
0.1
0.05
0.01
1
NIL
HORIZONTAL

PLOT
629
15
829
165
Number of positions
NIL
NIL
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -16777216 true
"Number of positions" 1.0 0 -16777216 true

@#$#@#$#@
WHAT IS IT?
-----------
This section could give a general understanding of what the model is trying to show or explain.


HOW IT WORKS
------------
This section could explain what rules the agents use to create the overall behavior of the model.


HOW TO USE IT
-------------
This section could explain how to use the model, including a description of each of the items in the interface tab.


THINGS TO NOTICE
----------------
This section could give some ideas of things for the user to notice while running the model.


THINGS TO TRY
-------------
This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.


EXTENDING THE MODEL
-------------------
This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.0.5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment11.19" repetitions="5" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <enumeratedValueSet variable="protect_for_workers">
      <value value="0.3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="toleration_ratio">
      <value value="0.8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="search_range">
      <value value="7"/>
      <value value="8"/>
      <value value="9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-workers">
      <value value="500"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="num-firms">
      <value value="50"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
